@@ -1,139 +1,94 @@
 testIndBeta = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL,robust=FALSE)
 {
   #   TESTINDBETA Conditional Independence Test based on beta regression for proportions
-  #
   #   provides a p-value PVALUE for the null hypothesis: X independent by target
   #   given CS. The pvalue is calculated by comparing a beta regression model based 
   #   on the conditioning set CS against a model containing both X and CS. 
   #   The comparison is performed through a chi-square test with some degrees 
   #   of freedom on the difference between the log-likelihoodss of the two models. 
   #   TESTINDBETA requires the following inputs:
-  
   #   target: a column vector containing the values of the target variable. 
   #   target must be an integer vector, with values between 0 and 1 
-  #
   #   dataset: a numeric data matrix containing the variables for performing
   #   the conditional independence test. There can be mixed variables, i.e. continous and or categorical
-  #
   #   xIndex: the index of the variable whose association with the target
   #   must be tested. Can be mixed variables, either continous or categorical
-  #
   #   csIndex: the indices of the variables to condition on. 
-  #
   #   this method returns: the pvalue PVALUE, the statistic STAT and a control variable FLAG.
   #   if FLAG == 1 then the test was performed succesfully 
-  #
-  #   Examples:
-  #      # Perform a conditional independence test on a toy example.
-  #      x = c(19, 38, 44, 45, 49, 65, 71, 75, 77, 80);
-  #      y = c(0.969, 0.402, 0.143, 0.888, 0.117, 0.861, 0.606, 0.384, 0.178, 0.241);
-  #      cs = c(28, 75, 68, 26, 66, 51, 16, 70, 12, 89);
-  #      results = testIndBeta(y, cbind(x,cs), 1, 2)
-  #
-  #
-  #   See also testIndFisher
-  #
   #   References:
   #   [1] Ferrari S.L.P., Cribari-Neto F. (2004). Beta Regression for  
   #   Modelling Rates and Proportions. Journal of Applied Statistics, 
   #   31(7): 799--815.
-  
-  #initialization
-  
   csIndex[which(is.na(csIndex))] = 0
-  
-  if( hash )
-  {
+  if ( hash )  {
     csIndex2 = csIndex[which(csIndex!=0)]
-    csindex2 = sort(csIndex2)
+    csIndex2 = sort(csIndex2)
     xcs = c(xIndex,csIndex2)
     key = paste(as.character(xcs) , collapse=" ");
-    if( !is.null(stat_hash[[key]]) )
-    {
+    if ( !is.null(stat_hash[[key]]) )  {
       stat = stat_hash[[key]];
       pvalue = pvalue_hash[[key]];
       flag = 1;
-      
       results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
       return(results);
     }
   }
-  
   #if the test cannot performed succesfully these are the returned values
   pvalue = log(1);
   stat = 0;
   flag = 0;
-  
   #if the xIndex is contained in csIndex, x does not bring any new
   #information with respect to cs
-  if( !is.na(match(xIndex,csIndex)) )
-  {
-    if( hash )#update hash objects
-    {
+  if ( !is.na(match(xIndex, csIndex)) ) {
+    if ( hash )  {   #update hash objects
       stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
       pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
     }
     results <- list(pvalue = log(1), stat = 0, flag = 1, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
   }
-  
   #check input validity
-  if(xIndex < 0 || csIndex < 0)
-  {
+  if (xIndex < 0 || csIndex < 0)  {
     message(paste("error in testIndBeta : wrong input of xIndex or csIndex"))
     results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
   }
-  
   #xIndex = unique(xIndex);
   #csIndex = unique(csIndex);
-  
   #extract the data
   x = dataset[ , xIndex];
   cs = dataset[ , csIndex];
-  
   #if x or target is constant then there is no point to perform the test
-  if( vara( as.numeric(x) ) == 0 )
-  {
-    if( hash )#update hash objects
-    {
-      stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
-    }
-    results <- list(pvalue = log(1), stat = 0, flag = 1, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
-    return(results);
-  }
-  
+  # if ( Rfast::Var( as.numeric(x) ) == 0 ) {
+  #   if ( hash )  {   #update hash objects
+  #     stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
+  #     pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
+  #   }
+  #   results <- list(pvalue = log(1), stat = 0, flag = 1, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
+  #   return(results);
+  # }
   #remove NAs-zeros from cs
   #csIndex = csIndex[csIndex!=0]
-  
   if( length(cs) == 0 || is.na(cs) )  cs = NULL;
-  
   #if x = any of the cs then pvalue = 1 and flag = 1.
   #That means that the x variable does not add more information to our model due to an exact copy of this in the cs, so it is independent from the target
-  if(length(cs)!=0)
-  {
-    if( is.null(dim(cs)[2]) ) #cs is a vector
-    {
-      if(any(x != cs) == FALSE)  #if(!any(x == cs) == FALSE)
-      {
-        if( hash )#update hash objects
-        {
-          stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
-          pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
+  if ( length(cs) != 0 )  {
+    if ( is.null(dim(cs)[2]) )  {    #cs is a vector
+      if (any(x != cs) == FALSE)  {      #if(!any(x == cs) == FALSE)
+        if ( hash )  {    #update hash objects
+          stat_hash[[key]] <- 0;     #.set(stat_hash , key , 0)
+          pvalue_hash[[key]] <- log(1);     #.set(pvalue_hash , key , 1)
         }
         results <- list(pvalue = log(1), stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
         return(results);
       }
-    }else{ #more than one var
-      for(col in 1:dim(cs)[2])
-      {
-        if(any(x != cs[,col]) == FALSE)  #if(!any(x == cs) == FALSE)
-        {
-          if( hash )#update hash objects
-          {
-            stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
-            pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
+    } else { #more than one var
+      for (col in 1:dim(cs)[2]) {
+        if (any(x != cs[,col]) == FALSE)  {    #if(!any(x == cs) == FALSE)
+          if ( hash )  {    #update hash objects
+            stat_hash[[key]] <- 0;    #.set(stat_hash , key , 0)
+            pvalue_hash[[key]] <- log(1);    #.set(pvalue_hash , key , 1)
           }
           results <- list(pvalue = log(1), stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
           return(results);
@@ -141,58 +96,45 @@ testIndBeta = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NU
       }
     }
   }
-   
-
   #trycatch for dealing with errors
   res <- tryCatch(
 {
   #if the conditioning set (cs) is empty, we use the t-test on the coefficient of x.
-  if(length(cs) == 0)
-  {
+  if ( length(cs) == 0 )  {
     #if the univariate models have been already compute
-    if(!is.null(univariateModels))
-    {
+    if ( !is.null(univariateModels) )   {
       pvalue = univariateModels$pvalue[[xIndex]];
       stat = univariateModels$stat[[xIndex]];
       flag = univariateModels$flag[[xIndex]];
-      
       results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
       return(results);
     }
-     #Fitting beta regressions
-      if ( is.null(wei) )  fit1 = Rfast::beta.mle(target)  else  fit1 <- betamle.wei(target, wei)
-      fit2 = beta.reg(target, x, wei = wei)
-  }else{
-      #Fitting beta regressions
-
-      fit1 = beta.reg( target, dataset[, csIndex], wei = wei )
-      fit2 = beta.reg(target, dataset[, c(csIndex, xIndex)] , wei = wei )  ;
+    #Fitting beta regressions
+    if ( is.null(wei) )  fit1 = Rfast::beta.mle(target)  else  fit1 <- betamle.wei(target, wei)
+    fit2 = beta.reg(target, x, wei = wei)
+  } else {
+    #Fitting beta regressions
+    fit1 = beta.reg( target, dataset[, csIndex], wei = wei )
+    fit2 = beta.reg(target, dataset[, c(csIndex, xIndex)] , wei = wei )  ;
   }
-      lik1 = fit1$loglik
-      lik2 = fit2$loglik
-      stat = 2 * abs(lik1 - lik2)
-      dof = length( fit2$be ) - length( fit1$be )
-      pvalue = pchisq(stat, dof, lower.tail = FALSE, log.p = TRUE) 
-  
+  stat = 2 * fit2$loglik - 2 * fit1$loglik
+  dof = length( fit2$be ) - length( fit1$be )
+  pvalue = pchisq(stat, dof, lower.tail = FALSE, log.p = TRUE) 
   #calculate the p value and stat.
   flag = 1;
   #update hash objects
-  if( hash )
-  {
+  if( hash )  {
     stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
     pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
   }
-  
   #last error check
-  if(is.na(pvalue) || is.na(stat))
-  {
+  if (is.na(pvalue) || is.na(stat) ) {
     pvalue = log(1);
     stat = 0;
     flag = 0;
-  }else{
+  } else {
     #update hash objects
-    if( hash )
-    {
+    if ( hash )  {
       stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
       pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
     }
@@ -200,26 +142,22 @@ testIndBeta = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NU
   #testerrorcaseintrycatch(4);
   results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
   return(results);
-  
 },
 error=function(cond) {
 #   message(paste("error in try catch of the testIndBeta test"))
 #   message("Here's the original error message:")
 #   message(cond)
 #   
-#   #for debug
-#     print("\nxIndex = \n");
-#     print(xIndex);
-#     print("\ncsindex = \n");
-#     print(csIndex);
-#   
-#   #error case
+#   for debug
+#   print("\nxIndex = \n");
+#   print(xIndex);
+#   print("\ncsindex = \n");
+#   print(csIndex);
+#   error case
+#  stop();
   pvalue = log(1);
   stat = 0;
   flag = 0;
-  
-  # stop();
-  
   results <- list(pvalue = pvalue, stat = stat, flag = flag, stat_hash=stat_hash, pvalue_hash=pvalue_hash);
   return(results);
 },
@@ -233,5 +171,4 @@ finally={}
   )
   
   return(res);
-  
 }
\ No newline at end of file
